/* libdf3 private header */

#ifndef DF3_LIBRARY_PRIV_H
#define DF3_LIBRARY_PRIV_H 201705L

// TODO test in 'configure.ac'.
#include <limits.h>
#if 8 != CHAR_BIT
#error "oops, need 8-bit char."
#endif

#include <df3.h>

#include <errno.h>
#include <sys/types.h>


/* .df3 file header, see man page */
#define DF3_HEADERSIZE  6

/* only mmap(2) files up to this limit.
 * if you have acres of RAM, you could increase this limit.
 */
#ifndef DF3_MMAPLIMIT
#define DF3_MMAPLIMIT 125000006L
#endif


uint16_t df3CalcChecksum(df3Handle const * const);
off_t df3CalcFilepointer(df3Handle *, df3Coord const);
int df3ExistFileCheck(char const * const, off_t *);
bool df3ValidAt(df3Axes const, df3Coord const);
bool df3ValidHandle(df3Handle const * const);

/* df3 files are big-endian */
typedef union endianHeader endianHeader;
union endianHeader {
  uint8_t raw[DF3_HEADERSIZE];
  uint16_t v[3];
};

typedef union endianData endianData;
union endianData {
  uint8_t raw[4];
  uint8_t v8;
  uint16_t v16;
  uint32_t v32;
};

/* smaller files (bpv 1 only) get mmapped */
typedef struct mmFile mmFile;
struct mmFile {
  int fd;
  uint8_t *dp;
  size_t n;
};

/* axes multiplied */
#define AXMUL(a_)  ((a_).x * (a_).y * (a_).z)

#endif

/* ----------------------------------------------------------------- *
 *  this file is part of 'libdf3', part of the 'df3 tools' project.  *
 *  the content above is covered by the GPLv3+, see file COPYING.    *
 *  copyright (c) 2017 jr <creature.eternal@gmail.com>.              *
 *  all rights reserved.                                             *
 * ----------------------------------------------------------------- */
